use crate::models::deck::{Card, Deck};
use std::collections::HashMap;

#[derive(Debug)]
pub enum HandSort {
    None,
    Value,
    Rank,
}

#[derive(Debug)]
pub struct Hand {
    pub cards: Vec<Card>,
    pub card_sort: HandSort,
}

impl Hand {
    pub fn new() -> Hand {
        Hand {
            cards: Vec::new(),
            card_sort: HandSort::None,
        }
    }

    pub fn draw(&mut self, count: u8, deck: &mut Deck) {
        for _ in 0..count {
            let card = deck.cards.pop().unwrap();
            self.cards.push(card);
        }
    }

    pub fn sort(&mut self, hand_sort: HandSort) {
        match hand_sort {
            HandSort::Value => self.sort_by_value(),
            HandSort::Rank => self.sort_by_rank(),
            HandSort::None => (),
        }
    }

    pub fn sort_by_value(&mut self) {
        self.cards.sort_by(|a, b| a.value.cmp(&b.value));
        self.card_sort = HandSort::Value;
    }

    pub fn sort_by_rank(&mut self) {
        //self.cards.sort_by(|a, b| a.rank.cmp(&b.rank));
        self.card_sort = HandSort::Rank;
    }

    fn pair_count(&self) -> u8 {
        let mut pairs = 0;

        let mut value_counts: HashMap<u8, u8> = HashMap::new();
        for card in self.cards.iter() {
            value_counts.insert(card.value, value_counts.get(&card.value).unwrap_or(&0) + 1);
        }

        for i in value_counts.values() {
            if i > &1 {
                pairs = pairs + 1;
            }
        }

        pairs
    }

    pub fn has_single_pair(&mut self) -> bool {
        if !matches!(self.card_sort, HandSort::Value) {
            self.sort(HandSort::Value);
        }

        if self.pair_count() == 1 {
            return true;
        }

        false
    }

    pub fn has_two_pair(&mut self) -> bool {
        if !matches!(self.card_sort, HandSort::Value) {
            self.sort(HandSort::Value);
        }

        if self.pair_count() == 2 {
            return true;
        }

        false
    }

    pub fn has_three_of_a_kind(&mut self) -> bool {
        if !matches!(self.card_sort, HandSort::Value) {
            self.sort(HandSort::Value);
        }
        for i in 2..self.cards.len() {
            if self.cards[i].rank == self.cards[i - 1].rank && self.cards[i].rank == self.cards[i - 2].rank {
                return true;
            }
        }

        false
    }

    pub fn has_straight(&mut self) -> bool {
        if !matches!(self.card_sort, HandSort::Value) {
            self.sort(HandSort::Value);
        }

        let hand_size = self.cards.len();

        // Handle Ace low straight
        if self.cards[0].value == 2 && self.cards[hand_size - 1].value == 14 {
            self.cards[hand_size - 1].value = 1;
            self.cards.rotate_right(1)
        }

        for i in 1..self.cards.len() {
            if self.cards[i].value != self.cards[i - 1].value + 1 {
                return false;
            }
        }

        true
    }

    pub fn has_flush(&self) -> bool {
        for i in 1..self.cards.len() {
            if self.cards[i].suit != self.cards[i - 1].suit {
                return false;
            }
        }

        true
    }

    pub fn has_full_house(&mut self) -> bool {
        if self.has_three_of_a_kind() && self.has_two_pair() {
            return true;
        }

        false
    }

    pub fn has_four_of_a_kind(&mut self) -> bool {
        if !matches!(self.card_sort, HandSort::Value) {
            self.sort(HandSort::Value);
        }
        for i in 3..self.cards.len() {
            if self.cards[i].rank == self.cards[i - 1].rank &&
                self.cards[i].rank == self.cards[i - 2].rank &&
                self.cards[i].rank == self.cards[i - 3].rank
            {
                return true;
            }
        }

        false
    }

    pub fn has_straight_flush(&mut self) -> bool {
        self.has_straight() && self.has_flush()
    }

    pub fn has_royal_flush(&mut self) -> bool {
        self.has_flush() && self.has_straight() && self.cards[0].value == 10
    }
}
